function [resSCV,resG2,fobjAC]=kpcfit_sub_acfit(E,SA,SAlags,J,MaxIterAC, MaxRunsAC, MaxResAC)
%% optimization parameters
MaxTimeAC = 10; % 10sec
TOL=1e-9; % numerical tolerance
EPSTOL=100*TOL;
MaxResAC=min([MaxResAC,MaxRunsAC]);
options = optimset('LargeScale','off', ...
    'MaxIter',MaxIterAC,  ...
    'MaxFunEvals',1e10,  ...
    'MaxSQPIter',500,  ...
    'TolCon',TOL,  ...
    'Display','off', ...
    'OutputFcn',@outfun);

%% other variables
SCV=(E(2)-E(1)^2)/E(1)^2;
NSA=norm(SA,2); % normalization constant for objective function
SCVJ=SCV; % initialize nnlcon variable

%% multi-start optimization
fset = [];   % record the objective function values for each param set
xparamset = {}; % record all results
xparam_best = [];
f_best = inf;
bestindex = 0;
for i = 1:MaxRunsAC
    x0=rand(2*J,1);
    tstart = tic();
    fprintf(1,'acfit: run %d of %d ',i,MaxRunsAC);
    tic;
    [x,f]=fmincon(@objfun,x0,[],[],[],[],x0*1+EPSTOL,[],@nnlcon,options);
    fprintf(1,'(%3.3f sec) ',toc);
    xparamset{end+1} = x;
    fset(1,end+1) = f;
    if f < f_best
        fprintf(1,'**best**',toc);
        f_best = f;
        xparam_best = x;
    end
    fprintf(1,'\n',toc);
end
[vl,ind] = sort(fset,2,'ascend');

resG2 = {};
resSCV = {};
fobjAC = [];
for i = 1:MaxResAC
    [SCVj,G2j]=xtopar(xparamset{ind(i)});
    G2j(find(G2j>1-EPSTOL))=1-EPSTOL;
    resSCV{end+1} = SCVj;
    resG2{end+1} = G2j;
    fobjAC(i) = fset(1,ind(i));
end

    function [SCVj,G2j]=xtopar(x)
        SCVj=x(1:J);
        G2j=x((J+1):end);
    end

    function [c,ceq]=nnlcon(x)
        [SCVj,G2j]=xtopar(x);
        c=[];
        % MAP(2) CONSTRAINTS
        c(end+1)=G2j(1)-1-EPSTOL;         % G2j(1) < 1
        c(end+1)=-(1-EPSTOL)-G2j(1);      % G2j(1) > 0
        c(end+1)=(0.5-EPSTOL)-SCVj(1);    % SCVj(1) > 0.5
        % HMAP(2) CONSTRAINTS
        for j=2:J
            c(end+1)=G2j(1)-G2j(j);       % G2j(j) > G2j(1) ?
            c(end+1)=G2j(j)-1-EPSTOL;     % G2j(j) < 1
            c(end+1)=1.1-SCVj(j);         % SCVj(j) > 1.1
            c(end+1)=EPSTOL-G2j(j);       % G2j(j) > 0
        end
        ceq=[];
        c(end+1)=(SCVJ-SCV)^2-(0.1*SCV)^2;
    end

    function f=objfun(x)
        [SCVj,G2j]=xtopar(x);
        [SCVJ,acfCoeff]=kpcfit_sub_eval_acfit(SCVj,G2j,SAlags);
        f=norm((SA-acfCoeff),2)/NSA;
    end

    function stop = outfun(x,optimValues,state)
        stop = false;
        if strcmpi(state,'iter')
            if mod(optimValues.iteration,MaxIterAC)==0 && optimValues.iteration>1
                if ( optimValues.fval >f_best)
                    stop = true;
                end
            end
            telapsed = toc(tstart);
            if ( telapsed>MaxTimeAC && optimValues.iteration > MaxIterAC)
                fprintf(1,'acfit: time limit reached in autocorrelation fitting\n');
                stop = true;
            end
        end
    end
end