function [MAP,subMAPs]=kpcfit_sub_compose(E1j,SCVj,E3j,G2j)
% map_kpcfit_compose make a large MAP out of several small MAP(2) or
% MA(1)s.
%
%  MAP=map_kpcfit_compose(E1j,SCVj,E3j,G2j) gives a big MAP composed by
%  many small MAP(2) or MAP(1)s with individual E1, SCV, E3, G2.
% 
%   Input: 
%       E1j     = a vector of the first moment for every map
%       SCVj    = a vector of squared coefficient of variation for every
%       map
%       E3j     = a vector of the third moment for every map
%       G2j     = a vector of gamma2 for every map
%
%   Output:
%       MAP     = A map composed by J maps, each map(2) has the specified
%       first moment, SCV, third moment, gamma2 in the vectors of E1j,
%       SCVj, E3j and so on. 
%
%   Comment: 
%       The first map is a mmpp2 while the other J-1 maps are HMAPs
%

J=length(G2j);
% compute the composed process
MAP=mmpp2_fit3(E1j(1),(1+SCVj(1))*E1j(1)^2,E3j(1),G2j(1));
subMAPs{1}=MAP;
for j=2:J
    MAPj=map_block(E1j(j),(1+SCVj(j))*E1j(j)^2,E3j(j),G2j(j));
    subMAPs{j}=MAPj;
    MAP= map_kpc(MAP,MAPj);
end
MAP=map_normalize(MAP);            
end
