function acfun=map_acf(MAP,lagset,OPTION)
if nargin==1
    acfun=map_acf(MAP,1);
elseif nargin==2
    % p=mapacf(MAP,lagset)
    lambda=1/map_moment(MAP,1);
    p=map_prob(MAP);
    e=ones(size(MAP{1},1),1);
    invD0=inv(-MAP{1});
    if strcmpi(class(MAP{1}(1,1)),'sym')
        acfun=sym([]);
    else
        acfun=[];
    end
    P=invD0*MAP{2};
    for lag=lagset
        acfun(end+1)=lambda*p*(P^lag)*invD0*e;
    end
    acfun=(acfun-1)./map_scv(MAP);
end
end
function SCV=map_scv(MAP)
% SCV=map_scv(MAP) - Compute squared coefficient of variation of
% inter-arrival times
%
%  Input:
%  MAP: a MAP in the form of {D0,D1}
%
%  Output:
%  SCV: squared coefficient of variation of inter-arrival times
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
SCV=(map_moment(MAP,2)-map_moment(MAP,1)^2)/map_moment(MAP,1)^2;
end
function PIQ=map_prob(MAP)
% PIQ=map_prob(MAP) - Equilibrium distribution of the underlying
% continuous-time process
%
%  Input:
%  MAP: a MAP in the form of {D0,D1}
%
%  Output:
%  PIQ: equilibrium distribution of the continuous-time Markov chain
%  Q=D0+D1
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
PIQ=ctmc_solve(MAP{1}+MAP{2});
end
function PROB=ctmc_solve(Q,options)
% PROB=ctmc_solve(P) - Equilibrium distribution of the continuous-time
% Markov chain
%
%  Input:
%  P: infinitesimal generator of the continuous-time Markov chain
%
%  Output:
%  PROB: equilibrium distribution of the continuous-time Markov chain
%
%  Examples:
%  - ctmc_solve([-0.5,0.5;0.2,-0.2])
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
Qsav=Q;
zerocol=find(sum(abs(Q))==0);
if length(zerocol)>1
    warning('ctmc_solve: the infinitesimal generator is reducible');
    b=zeros(size(Q,1),1);
elseif length(zerocol)==0
    zerocol=size(Q,1);
end
b=zeros(size(Q,1),1); b(zerocol)=1;
Q(:,zerocol)=1; % add normalization condition
p=(Q')\b; % compute steady state probability vector
%p=bicgstab(Q',b); % compute steady state probability vector
PROB=p'; % p is a row vector
end
function PROB=dtmc_solve(P)
% PROB=dtmc_solve(P) - Equilibrium distribution of a discrete-time Markov
% chain
%
%  Input:
%  P: stochastic transition matrix of the discrete-time Markov chain
%
%  Output:
%  PROB: equilibrium distribution of the discrete-time Markov chain
%
%  Examples:
%  - dtmc_solve([0.5,0.5;0.2,0.8])
%
% MAP Queueing Networks Toolbox
% Version 1.0 	 15-Apr-2008
PROB=ctmc_solve(P-eye(size(P)));
end
