function ED=map_moment(MAP,iset)
% MAP_MOMENT computes moments of a MAP
%
%   E=map_moment(MAP,i) returns moment of order i of inter-arrival times
%   E=map_moment(MAP,v), v=[i,j,..], returns vector of moments of order i,j,... of inter-arrival times
%
%   Input:
%       MAP = a MAP in the form of {D0,D1}
%         v = a vector of moments orders
%
%   Output:
%       ED = a row vector of moments
%

D0=MAP{1};
D1=MAP{2};
ED=[MAP{1}(1,1)];
e=ones(length(D0),1);
for i=iset
    A=(inv(-D0)^i) ;
    pi=map_prob(MAP);
    ED(end+1)=factorial(i)*pi*D1*A*e/(pi*D1*e);
end
ED(1)=[];

    function p=map_prob(MAP)
        % FUNCTION p=map_prob(MAP) - equilibrium probabilities of CTMC underlying MAP        
        Q=MAP{1}+MAP{2};
        p=ctmc_solve(Q);
        
        function p=ctmc_solve(Q)
            % FUNCTION p=ctmc_solve(Q) - solve ctmc by global balance
            zerocol=find(sum(abs(Q))==0);
            if length(zerocol)>1
                warning('two cols of Q are composed by zeros only.');
                b=zeros(size(Q,1),1);
            elseif length(zerocol)==0
                zerocol=size(Q,1);
            end
            b=zeros(size(Q,1),1); b(zerocol)=1;
            Q(:,zerocol)=1; % add normalization condition
            p=(Q')\b; % compute steady state probability vector
            p=p'; % p is a row vector
        end
        
    end
end
