function [MAP]=map_normalize(MAP,TOL)
%  map_normalize normalizes a MAP according to an input precision.
%
%   [MAP] = map_normalize(MAP,TOL) returns the normalized MAP
%
%   Input:
%       MAP = a map in the form of {D0,D1}
%       TOL = precision, it's the optional parameter, by default it's
%             eps
%
%
%   Output:
%       MAP = a normalized MAP according to the precision, in the sense
%             that the sum of every row is less than eps
if nargin==1
    TOL=eps;
end
nStates=size(MAP{1},1);
nMAP=length(MAP);

MAP{1}(find(abs(MAP{1})<TOL & MAP{1}<0))=0;
MAP{2}(find(abs(MAP{2})<TOL & MAP{2}<0))=0;
for n=1:nStates
    % recalculate the diagonal elements in D0 according to the other
    % elements in the same row
    MAP{1}(n,n)=0;
    for b=1:nMAP
        MAP{1}(n,n)=MAP{1}(n,n)-sum(MAP{b}(n,:));
    end
end
for i=1:nStates
    for j=1:nStates
        % if there is any complex element, replace them with their real
        % parts
        MAP{1}(i,j)=real(MAP{1}(i,j));
        MAP{2}(i,j)=real(MAP{2}(i,j));
    end
end

end